#include "SpellShot.h"

#include <math.h>

const float accelTime = 0.6f;
const float maxVel = 300;
const float defaultLife = 8.0;

SpellShot* shot = 0;
FrameSet magicFrames;
const Frame* genericShotFrame;
const Frame* shotFrame[eNumSpellSources];
ParticleEmitter* shotEmitter[eNumSpellSources];


SpellShot::SpellShot(float x, float y, float vx, float vy, const Frame* f, ParticleEmitter* e, eSpellSources t, const SpellPoints& pts) :
	points(pts)
{
	pos[0] = x;
	pos[1] = y;
	vel[0] = vx;
	vel[1] = vy;
	accel[0] = 0;
	accel[1] = 0;
	accelMag = 0;

	frame = f;
	emitter = e;
	type = t;

	size = 1.0;

	life = defaultLife;
}

SpellShot::~SpellShot()
{
	// delete emitter;
	emitter->Kill();
}

void SpellShot::Draw()
{
	frame->Draw(pos[0],pos[1],0,size,size,1.0,1.0,1.0,1.0,false,false);
	// emitter->Draw();
}

void SpellShot::Update(double timeDif)
{
	// Update position
	pos[0] += timeDif*vel[0];
	pos[1] += timeDif*vel[1];

	// Update velocity 
	vel[0] += timeDif*accel[0]*accelMag;
	vel[1] += timeDif*accel[1]*accelMag;

	// Adjust max velocity
	float mag = sqrt(vel[0] * vel[0] + vel[1] * vel[1]);
	if (mag > maxVel)
	{
		vel[0] *= maxVel/mag;
		vel[1] *= maxVel/mag;
	}


	accelMag -= timeDif / accelTime;
	accelMag = max(accelMag,0);

	emitter->x = pos[0];
	emitter->y = pos[1];
	emitter->z = 0;

	life -= timeDif;
}


void SpellShot::Manipulate(float x, float y, float timeDif)
{
	float mag = 3.0;

	// Update velocity 
	vel[0] += timeDif*(x - pos[0])*mag;
	vel[1] += timeDif*(y - pos[1])*mag;

	/* accel[0] = (x - pos[0])*3;
	accel[1] = (y - pos[1])*3;

	accelMag = 1.0;*/
}

void SpellShot::ResetEmitter(ParticleEmitter* e)
{
	emitter->Kill();
	emitter = e;
}

int SpellShot::calcDamage()
{
	return points.getFirePts() + points.getWaterPts() + points.getEarthPts() + points.getElecPts();
}
