#include "enum.h"
#include "bbox.h"
#include "clouds.h"
#include "Game.h"
#include "particle.h"
#include "SpellShot.h"
#include "stroke.h"
#include "Timer.h"
#include "wizard.h"
#include "spellSource.h"
#include "math.h"

#include "SoundManager.h"

#include <stdlib.h>
#include <time.h>
#include <windows.h>

#include <iostream>
#include <map>
#include <set>
#include <string>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glpng.h>
#include "glfont.h"
#include "SDL.h"

using namespace std;


static void LoadData();
static void SetupGL();
static void SetupGame();

static void MessageLoop();
static void HandleMouse(SDL_Event& e);
static void HandleMouseMotion(SDL_Event& e);

static void Draw();
static void Update(double timeDif);

void NextPlayer();


static GLenum getGlFormat(int nComponents);

double timeLast;
double timeCur;
int dragStartX, dragStartY;
double velX = 1.0, velY = 0.0;
double posX = 0.0, posY = 0.0;

int mousePos[2];
int mouseVel[2];

typedef std::map<std::string,AudioSample> SoundMap;
typedef SoundMap::iterator SoundIt;
SoundMap soundMap;

typedef std::map<std::string, eSpellSources> GestureMap;
typedef GestureMap::iterator GestureIt;
GestureMap gestureMap;

// static pngRawInfo imgBack;
static GLuint texBack;
static GLuint texParticle;

Clouds* clouds;

ParticleEmitter* wandEmitter = 0;

// chasm bbox
//
BBox upperBoard(512, 150, 1024, 300);
BBox chasm(512, 375, 1024, 150);
BBox lowerBoard(512, 609, 1024, 318);
//
// chasm bbox


// spell source stuff
//
typedef std::multimap<float, SpellSource*> SpellSourceCont;
typedef SpellSourceCont::iterator          SpellSourceIt;

FrameSet*              spellSourcesFS = 0;
AnimationSet*          spellSourcesAS = 0;

eSpellSources          spellSourceToCreate;
SpellSourceCont        spellSources;
std::set<SpellSource*> spellSourcesUsed;
//
// spell source stuff


// wizard stuff
//
FrameSet*     wizardFS = 0;
AnimationSet* wizardAS = 0;

Wizard* wizard1 = 0;
Wizard* wizard2 = 0;

Wizard* curWizard   = 0;
Wizard* otherWizard = 0;
//
// wizard stuff

int gestureAttempts = 0;
const int maxGestureAttempts = 6;

enum State
{
	Gesture, Aim, Shot, Create, Walk, Death, Victory
};

State state;


SpellPoints shotPoints[eNumSpellSources];

PixelPerfectGLFont font;
Timer updateTimer, frameTimer, fpsTimer;
double fps;

const unsigned int HEIGHT = 768;
const unsigned int WIDTH = 1024;


FrameSet hudfs;

const Frame* healthHudFrame;
const Frame* manaHudFrame;
const Frame* fireHudFrame;
const Frame* waterHudFrame;
const Frame* earthHudFrame;
const Frame* elecHudFrame;

FrameSet helpFS;
const Frame* helpFrame;

FrameSet victoryFS;
const Frame* victoryFrame;


void DisplayTitle()
{
	pngRawInfo imgTitle;
	pngLoadRaw("art/title.png", &imgTitle);

	// Flip image
	int pitch = imgTitle.Width * (imgTitle.Depth*imgTitle.Components / 8);
	unsigned char* rowPix = new unsigned char[pitch];
	// 
	for (int y = 0; y < imgTitle.Height/2; y++)
	{
		memcpy(rowPix, imgTitle.Data+y*pitch, pitch);
		memcpy(imgTitle.Data+y*pitch, imgTitle.Data+(imgTitle.Height-1-y)*pitch, pitch);
		memcpy(imgTitle.Data+(imgTitle.Height-1-y)*pitch, rowPix, pitch);
	}
	delete [] rowPix;

	GLenum format = getGlFormat(imgTitle.Components);
	// glPixelZoom(1.0, -1.0);
	// glRasterPos2f(-1.0, 1.0);
	glPixelStorei(GL_PACK_ALIGNMENT, 1);

	// Wait till button pressed
	SDL_Event e;
	bool stop = false;
	while (!stop)
	{
		// Process events on screen
		while(SDL_PollEvent(&e)) {

			switch( e.type ) {
			case SDL_MOUSEBUTTONUP:
			case SDL_KEYUP:
				stop = true;
				break;
			case SDL_QUIT:
				exit(0);
				break;
			}
		}

		glClear(GL_COLOR_BUFFER_BIT);
		glDrawPixels(imgTitle.Width, imgTitle.Height, format, GL_UNSIGNED_BYTE, imgTitle.Data);
		SDL_GL_SwapBuffers();
		SDL_Delay(1);
	}


	

	// Free image
	free(imgTitle.Data);
	free(imgTitle.Palette);
}

int main(int argc, char *argv[]) {
    
    /* Initialize defaults, Video and Audio */
    if((SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_TIMER)==-1)) { 
        printf("Could not initialize SDL: %s.\n", SDL_GetError());
        exit(-1);
    }

	SoundManager::Singleton->Activate();
	SoundManager::Singleton->SetNumChannels(32);


	stroke_init();


	SetupGL();

	LoadData();

	SetupGame();


	DisplayTitle();

	MessageLoop();

    
    /* Shutdown all subsystems */
    SDL_Quit();

	SoundManager::Singleton->Deactivate();
    

    exit(0);
}


void SetupGL()
{
	const SDL_VideoInfo* vi;

	// int w = 1024, h = 768;
	int bpp, flags;

	vi = SDL_GetVideoInfo();
	bpp = vi->vfmt->BitsPerPixel;

/*    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );*/

    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 8 );
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 8 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 8 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 32 );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );


	// flags = SDL_OPENGL; //| SDL_FULLSCREEN
	flags = SDL_OPENGL | SDL_FULLSCREEN;

	if(SDL_SetVideoMode(WIDTH, HEIGHT, bpp, flags ) == 0) {
        fprintf( stderr, "Video mode set failed: %s\n", SDL_GetError( ) );
        exit(-1);
    }
	glMatrixMode(GL_PROJECTION);
	glViewport(0,0,WIDTH,HEIGHT);
	glOrtho(0,WIDTH,HEIGHT,0,-1,1);

	glMatrixMode(GL_MODELVIEW);
}

void MessageLoop()
{

	AudioSample as1,as2,as3;
	as1.LoadWave("sounds/ding.wav");
	as2.LoadWave("sounds/chord.wav");
	as3.LoadWave("sounds/chimes.wav");

	GLuint textureName;
	glGenTextures(1, &textureName);
	try { font.Create("art/arial20.glf", textureName); }
	catch(GLFontError::InvalidFile) { cout << "Cannot load font\n"; }


	updateTimer.Reset();
	frameTimer.Reset();
	fpsTimer.Reset();

	double minUpdateTime = 1.0/100.0;
	double minFrameTime = 1.0/60.0;
	SDL_Event e;
	unsigned int lastTicks = SDL_GetTicks();
	unsigned int framecount = 0;
	while (1)
	{
		// Process events on screen
		while(SDL_PollEvent(&e)) {

			switch( e.type ) {
			case SDL_KEYUP:
				if (e.key.keysym.sym == SDLK_ESCAPE)
					return;
				break;
			case SDL_QUIT:
				return;
				break;
			case SDL_MOUSEBUTTONDOWN:
				{
					//SoundManager::Singleton->Play(as1,0);
					vector<AudioSample*> v;
					v.push_back(&as1);
					v.push_back(&as2);
					v.push_back(&as3);
					//SoundManager::Singleton->PlayQueue(v);

				}
			case SDL_MOUSEBUTTONUP:
				HandleMouse(e);
				break;
			case SDL_MOUSEMOTION:
				HandleMouseMotion(e);
				break;
			}

		}
		if (updateTimer.Elapsed() > minUpdateTime)
		{
			Update(updateTimer.Elapsed());

			//do state update
			updateTimer.Reset();
		}
		if (frameTimer.Elapsed() > minFrameTime)
		{
			// Draw
			Draw();
			framecount++;
			if (fpsTimer.Elapsed() >= 2.0)
			{
				fps = (double)framecount / fpsTimer.Elapsed();
				fpsTimer.Reset();
				framecount = 0;
				//cout << "fps " << fps << endl;
			}
			frameTimer.Reset();
		}
		SDL_Delay(1);
	}
}


void HandleMouseMotion(SDL_Event& e)
{
	mousePos[0] = e.motion.x;
	mousePos[1] = e.motion.y;
	mouseVel[0] = e.motion.xrel;
	mouseVel[1] = e.motion.yrel;
	
	if( (e.motion.state & SDL_BUTTON(1)) && (state == Gesture) )
	{
		stroke_record(e.motion.x, e.motion.y);
	}
}



eSpellSources ProcessGesture(const char* str)
{	
	GestureIt it = gestureMap.find(str);


	// Check if successful gesture
	if (it == gestureMap.end())
	{
		gestureAttempts++;

		if (gestureAttempts > maxGestureAttempts)
		{
			gestureAttempts = 1;
		}

		char sound[256];
		sprintf(sound, "badgesture%d",gestureAttempts);
		PlaySoundEffect(sound);

		return eNumSpellSources;
	}
	
	eSpellSources sType = it->second;


	// Check if have enough mana
	if (costSpell[sType] > curWizard->getMana())
	{
		PlaySoundEffect("needmana");
		return eNumSpellSources;
	}

	float sise = 1;

	const float sizeIncrease = 1.3;

	SpellPoints& pts = curWizard->getSpellPts();
	SpellPoints sPts = shotPoints[sType];
	switch( sType )
	{
		case eFire:
		case eWater:
		case eEarth:
		case eElec:
		case eGeneric:
		{
			if ( sType == eFire ) {
				if (pts.getFirePts() >= 1) {
					curWizard->decMana(costSpell[sType]);
					pts.decFirePts(1);
					while (pts.getFirePts())
					{
						sPts += shotPoints[sType];
						pts.decFirePts(1);
						sise*=sizeIncrease;
					}
				} else {
					PlaySoundEffect("needmana");
					return eNumSpellSources;
				}
			} else if ( sType == eWater ) {
				if (pts.getWaterPts() >= 1) {
					curWizard->decMana(costSpell[sType]);
					pts.decWaterPts(1);
					while (pts.getWaterPts())
					{
						sPts += shotPoints[sType];
						pts.decWaterPts(1);
						sise*=sizeIncrease;
					}
				} else {
					PlaySoundEffect("needmana");
					return eNumSpellSources;
				}
			} else if ( sType == eElec ) {
				if (pts.getElecPts() >= 1) {
					curWizard->decMana(costSpell[sType]);
					pts.decElecPts(1);
					while (pts.getElecPts())
					{
						sPts += shotPoints[sType];
						pts.decElecPts(1);
						sise*=sizeIncrease;
					}
				} else {
					PlaySoundEffect("needmana");
					return eNumSpellSources;
				}
			} else if ( sType == eEarth ) {
				if (pts.getEarthPts() >= 1) {
					curWizard->decMana(costSpell[sType]);
					pts.decEarthPts(1);
					while (pts.getEarthPts())
					{
						sPts += shotPoints[sType];
						pts.decEarthPts(1);
						sise*=sizeIncrease;
					}
				} else {
					PlaySoundEffect("needmana");
					return eNumSpellSources;
				}
			}

			ParticleEmitter* em = shotEmitter[sType]->Clone();
			float wx, wy;
			curWizard->getPosition(wx, wy);
			em->x = wx;
			em->y = wy;
			em->z = 0;
			emitterSet.Add(em);
			shot = new SpellShot(wx, wy - 32, velX, velY, shotFrame[sType], em, sType, sPts);
			shot->size = sise;
			spellSourcesUsed.clear();			

			state = Aim;
			break;
		}

		case eCreateFire:
		case eCreateWater:
		case eCreateEarth:
		case eCreateElec:
		{

			spellSourceToCreate = sType;
			state = Create;
			wandEmitter = shotEmitter[sType - eNumSpellSources]->Clone();
			emitterSet.Add(wandEmitter);
			break;
		}
	}

	PlaySoundQueue("s", str);

	return sType;
}


void NextPlayer()
{
	state = Gesture;
	gestureAttempts = 0;
	curWizard->resetHasWalked();
	if (curWizard == wizard1)
	{
		curWizard = wizard2;
		otherWizard = wizard1;
	}
	else
	{
		curWizard = wizard1;
		otherWizard = wizard2;
	}

	curWizard->incMana(MANA_PER_TURN);
}

void HandleMouse(SDL_Event& e)
{
	char str[256] = "";

	if (e.button.button == SDL_BUTTON_LEFT) {
		if (state == Gesture)
		{
			if ( e.type == SDL_MOUSEBUTTONDOWN )
			{
				stroke_trans(str);
				stroke_record(e.button.x, e.button.y);
			} else if ( e.type == SDL_MOUSEBUTTONUP ) {
				stroke_trans(str);
				eSpellSources sst = ProcessGesture(str);
				//printf("%s\n", eSpellSourcesStr[sst]);
			}
		} else if (state == Aim) {
			if ( e.type == SDL_MOUSEBUTTONDOWN )
			{
				// Set velocity
				float wx,wy;
				curWizard->getPosition(wx, wy);
				float velX = e.button.x - wx;
				float velY = e.button.y - wy;

				if (shot)
				{
					shot->vel[0] = velX;
					shot->vel[1] = velY;
				}

				char sound[256];
				sprintf(sound, "%sshot", eSpellSourcesStr[shot->type]);
				PlaySoundEffect(sound);

				int taunt = rand() % 4;
				if (taunt == 0)
					PlaySoundEffect("w_taunt1");
				else if (taunt == 1)
					PlaySoundEffect("w_taunt2");

				state = Shot;
			}
		} else if (state == Shot) {
			if ( e.type == SDL_MOUSEBUTTONDOWN )
			{
				// Move shot
				// shot->Manipulate(e.button.x, e.button.y);
			}
		} else if( state == Create ) {
			if( e.type == SDL_MOUSEBUTTONUP )
			{
				if( !chasm.pointInBox(e.button.x, e.button.y) )
				{
					// create spellSource
					//
					SpellSource* ss = SpellSource::createSpellSource(spellSourceToCreate, e.button.x, e.button.y);

					// create associated sprite
					//
					char str[32];
					if( strcmp(eSpellSourcesStr[ss->type()], "earth") == 0 )
					{
						if( rand() % 2 )
							sprintf(str, "earth1");
						else
							sprintf(str, "earth2");
					}
					else
					{
						sprintf(str, "%s", eSpellSourcesStr[ss->type()]);
					}

					Sprite* sp = new Sprite(*spellSourcesAS);
					sp->PlayAnimation(str);
					sp->SetNextAnimation(str);
					ss->addSprite(0, sp);

					// add to multimap sorted by y (for proper drawing)
					//
					float x, y;
					ss->getPosition(x, y);
					pair<float, SpellSource*> ssp;
					ssp.first  = y;
					ssp.second = ss;
					spellSources.insert(ssp);

					// Play sound
					char sound[256];
					sprintf(sound, "%ssource", eSpellSourcesStr[ss->type()]);
					PlaySoundEffect(sound);

					wandEmitter->Kill();
					wandEmitter = 0;

					NextPlayer();
				}
			}
		}
	} else if (e.button.button == SDL_BUTTON_RIGHT) {
		if( state == Gesture )
		{
			if( e.type == SDL_MOUSEBUTTONDOWN )
			{
				if( (curWizard == wizard1 && upperBoard.pointInBox(e.button.x, e.button.y) && !curWizard->hasWalked()) ||
				    (curWizard == wizard2 && lowerBoard.pointInBox(e.button.x, e.button.y) && !curWizard->hasWalked()) )
				{
					state = Walk;

					curWizard->setMoveTo(e.button.x, e.button.y);

					Sprite* sp = curWizard->getSprite(0);
					sp->PlayAnimation("walk");
					sp->SetNextAnimation("walk");

					PlaySoundEffect("step");

					float x, y;
					curWizard->getDir(x, y);
					if( x < 0.0f )
						sp->FlipLR(true);
					else
						sp->FlipLR(false);
				}
			}
		}
	}
}


void LoadSound(const char* name, const char* wav)
{
	soundMap[name].LoadWave(wav);
}

void PlaySoundEffect(const char* name)
{
	SoundIt it = soundMap.find(name);
	
	if (it != soundMap.end())
		SoundManager::Singleton->Play(it->second,0);
}

void PlaySoundQueue(const char* name, const char* comb)
{
	vector<AudioSample*> v;
	for (const char* c = comb; *c != '\0' ; c++)	
	{
		char str[256];
		sprintf(str, "%s%c", name, *c);
		SoundIt it = soundMap.find(str);
		if (it != soundMap.end())
			v.push_back(&it->second);
	}

	if (!v.empty())
		SoundManager::Singleton->PlayQueue(v);
}

void LoadSounds()
{
	// LoadSound("ding", "sounds/ding.wav");

	// Spell sounds
	LoadSound("s0", "sounds/s_0.wav");
	LoadSound("s1", "sounds/s_1.wav");
	LoadSound("s2", "sounds/s_2.wav");
	LoadSound("s3", "sounds/s_3.wav");
	LoadSound("s4", "sounds/s_4.wav");
	LoadSound("s5", "sounds/s_5.wav");
	LoadSound("s6", "sounds/s_6.wav");
	LoadSound("s7", "sounds/s_7.wav");
	LoadSound("s8", "sounds/s_8.wav");
	LoadSound("s9", "sounds/s_9.wav");

	// Spell source sounds
	LoadSound("firesource", "sounds/firesource.wav");
	LoadSound("watersource", "sounds/watersource.wav");
	LoadSound("earthsource", "sounds/rocksource.wav");
	LoadSound("elecsource", "sounds/electricsource.wav");


	// Wizard sounds
	LoadSound("needmana", "sounds/needmana.wav");

	LoadSound("w_miss", "sounds/w_bah.wav");

	LoadSound("w_taunt1", "sounds/w_takethat.wav");
	LoadSound("w_taunt2", "sounds/w_huzzah.wav");

	LoadSound("w_hit1", "sounds/w_dah.wav");
	LoadSound("w_hit2", "sounds/w_myeyes.wav");
	LoadSound("w_hit3", "sounds/w_ow.wav");
	LoadSound("w_hit4", "sounds/w_thathurt.wav");


	LoadSound("w_die1", "sounds/w_ahhhhh.wav");
	LoadSound("w_die2", "sounds/w_die.wav");

	LoadSound("victory1", "sounds/dodododo.wav");
	LoadSound("victory2", "sounds/dododododadaa.wav");
	// LoadSound("victory3", "sounds/tada.wav");


	LoadSound("fireshot", "sounds/shot3.wav");
	LoadSound("watershot", "sounds/shot1.wav");
	LoadSound("elecshot", "sounds/shot4.wav");
	LoadSound("genericshot", "sounds/shot2.wav");
	LoadSound("earthshot", "sounds/shot5.wav");

	LoadSound("hit1", "sounds/hit.wav");
	LoadSound("hit2", "sounds/loudhit.wav");

	LoadSound("fizzle", "sounds/splat.wav");
	

	LoadSound("badgesture1", "sounds/w_dah.wav");
	LoadSound("badgesture2", "sounds/notagesture.wav");
	LoadSound("badgesture3", "sounds/notagesture.wav");
	LoadSound("badgesture4", "sounds/pressh.wav");
	LoadSound("badgesture5", "sounds/readreadme.wav");
	LoadSound("badgesture6", "sounds/readit.wav");
	LoadSound("badgesture7", "sounds/readit.wav");

	LoadSound("step", "sounds/step.wav");
/*
	w_ahhhhh.wav
	w_bah.wav
	w_dah.wav
	w_die.wav
	w_errrrah.wav
	w_huzzah.wav
	w_laugh.wav
	w_myeyes.wav
	w_ow.wav
	w_takethat.wav
	w_thathurt.wav
	w_yah.wav
	w_yerah.wav*/
}

void LoadData()
{
	hudfs.LoadFromXML("art/HUDframes.xml");

	texBack = pngBind("art/background.png", PNG_NOMIPMAP, PNG_SOLID, 0, GL_CLAMP, GL_LINEAR, GL_LINEAR);
	texParticle = pngBind("art/particle.png", PNG_NOMIPMAP, PNG_ALPHA, 0, GL_CLAMP, GL_LINEAR, GL_LINEAR);

	magicFrames.LoadFromXML("art/magicFrames.xml");
	genericShotFrame = magicFrames.GetFrame("genericMagic");

	// load spell source data
	//
	spellSourcesFS = new FrameSet();
	spellSourcesFS->LoadFromXML("art/sourceFrames.xml");

	spellSourcesAS = new AnimationSet(*spellSourcesFS);
	spellSourcesAS->LoadFromXML("art/sourceAnim.xml");


	// load wizard data
	//
	wizardFS = new FrameSet();
	wizardFS->LoadFromXML("art/wizardFrames.xml");

	wizardAS = new AnimationSet(*wizardFS);
	wizardAS->LoadFromXML("art/wizardAnimSet.xml");

	victoryFS.LoadFromXML("art/victoryFrames.xml");


	// Create spell types
	shotFrame[eFire] = magicFrames.GetFrame("fireMagic");
	shotFrame[eWater] = magicFrames.GetFrame("waterMagic");
	shotFrame[eEarth] = magicFrames.GetFrame("earthMagic");
	shotFrame[eElec] = magicFrames.GetFrame("lightningMagic");
	shotFrame[eGeneric] = magicFrames.GetFrame("genericMagic");

	float colFire1[3] = {0.7f, 0.7f, 0};
	float colFire2[3] = {1.0f, 0.0f, 0};

	float colWater1[3] = {0.0f, 0.5f, 1.0f};
	float colWater2[3] = {1.0f, 1.0f, 1.0f};

	// float colEarth1[3] = {0.2f, 0.2f, 0.2f};
	// float colEarth2[3] = {0.7f, 0.7f, 0.7f};
	float colEarth1[3] = {0.7f, 0.4f, 0.0f};
	float colEarth2[3] = {0.7f, 0.7f, 0.0f};

	float colElec1[3] = {1.0f, 1.0f, 0};
	float colElec2[3] = {0.0f, 0.5f, 1.0f};

	float colGen1[3] = {1.0f, 1.0f, 1.0f};
	float colGen2[3] = {1.0f, 0.0f, 1.0f};

	
	shotEmitter[eFire] = new ParticleEmitter(texParticle, 32, 32, 0.03, 1.0, colFire1, colFire2);
	shotEmitter[eWater] = new SpinEmitter(texParticle, 32, 32, 0.02, 1.0, colWater1, colWater2, 30.0, 3.0);
	shotEmitter[eEarth] = new ParticleEmitter(texParticle, 16, 16, 0.02, 1.0, colEarth1, colEarth2);
	shotEmitter[eElec] = new SpinEmitter(texParticle, 32, 32, 0.02, 1.0, colElec1, colElec2, 30.0, 3.0);
	shotEmitter[eGeneric] = new ParticleEmitter(texParticle, 32, 32, 0.03, 1.0, colGen1, colGen2);


	// Load interface images
	healthHudFrame = hudfs.GetFrame("healthIcon");
	manaHudFrame = hudfs.GetFrame("manaIcon");
	fireHudFrame = hudfs.GetFrame("fireIcon");
	waterHudFrame = hudfs.GetFrame("waterIcon");
	earthHudFrame = hudfs.GetFrame("earthIcon");
	elecHudFrame = hudfs.GetFrame("electricIcon");

	// Load help
	helpFS.LoadFromXML("art/helpFrames.xml");
	helpFrame = helpFS.GetFrame("help");

	victoryFrame = victoryFS.GetFrame("victory");

	LoadSounds();
}


void AddGesture(const char* str, eSpellSources type)
{
	gestureMap.insert(std::make_pair(std::string(str), type));
}

void SetupGame()
{
	// Seed randomizer
	srand(time(NULL));
	rand(); rand(); rand();

	// Setup gesture map
	AddGesture("852",       eGeneric);
	AddGesture("74123",     eFire);
	AddGesture("1475963",   eWater);
	AddGesture("3214789",   eEarth);
	AddGesture("1474123",   eElec);
	AddGesture("95123",     eCreateFire);
	AddGesture("123658",    eCreateWater);
	AddGesture("321456987", eCreateEarth);
	AddGesture("1478963",   eCreateElec);

	// Setup shot points
	shotPoints[eFire].setFirePts(DEF_SS_SHOT_F);
	shotPoints[eWater].setWaterPts(DEF_SS_SHOT_W);
	shotPoints[eEarth].setEarthPts(DEF_SS_SHOT_E);
	shotPoints[eElec].setElecPts(DEF_SS_SHOT_L);

	timeLast = SDL_GetTicks() / 1000.0f;
	float col1[3] = {0.7f, 0.7f, 0};
	float col2[3] = {1.0f, 0.0f, 0};
	// emitter = new ParticleEmitter(texParticle, 32, 32, 0.03, 1.0, col1, col2);
	// emitter = new SpinEmitter(texParticle, 32, 32, 0.02, 1.0, col1, col2, 30.0, 3.0);
	// emitterSet.Add(emitter);
	clouds = new Clouds();


	// create a random source on either side
	//
	float xmin, xmax, ymin, ymax;
	chasm.getMin(xmin, ymin);
	chasm.getMax(xmax, ymax);


	eSpellSources randSS = eSpellSources(1 + rand() % (int(eNumSpellSources)-1));
	SpellSource* ss = SpellSource::createSpellSource(randSS, (WIDTH/5)+(rand()%(3*WIDTH/5)), (int(ymin)/4)+(rand()%(int(ymin)/2)));

	// create associated sprite
	//
	char str[32];
	if( strcmp(eSpellSourcesStr[ss->type()], "earth") == 0 )
	{
		if( rand() % 2 )
			sprintf(str, "earth1");
		else
			sprintf(str, "earth2");
	}
	else
	{
		sprintf(str, "%s", eSpellSourcesStr[ss->type()]);
	}

	Sprite* sp = new Sprite(*spellSourcesAS);
	sp->PlayAnimation(str);
	sp->SetNextAnimation(str);
	ss->addSprite(0, sp);

	// add to multimap sorted by y (for proper drawing)
	//
	float x, y;
	ss->getPosition(x, y);
	pair<float, SpellSource*> ssp;
	ssp.first  = y;
	ssp.second = ss;
	spellSources.insert(ssp);


	randSS = eSpellSources(1 + rand() % (int(eNumSpellSources)-1));
	ss = SpellSource::createSpellSource(randSS, (WIDTH/5)+(rand()%(3*WIDTH/5)), (int(ymax)+((HEIGHT-int(ymax))/5))+(rand()%(3*(HEIGHT-int(ymax))/5)));

	// create associated sprite
	//
	str[32];
	if( strcmp(eSpellSourcesStr[ss->type()], "earth") == 0 )
	{
		if( rand() % 2 )
			sprintf(str, "earth1");
		else
			sprintf(str, "earth2");
	}
	else
	{
		sprintf(str, "%s", eSpellSourcesStr[ss->type()]);
	}

	sp = new Sprite(*spellSourcesAS);
	sp->PlayAnimation(str);
	sp->SetNextAnimation(str);
	ss->addSprite(0, sp);

	// add to multimap sorted by y (for proper drawing)
	//
	x, y;
	ss->getPosition(x, y);
	ssp.first  = y;
	ssp.second = ss;
	spellSources.insert(ssp);



	// create wizards
	//
	wizard1 = new Wizard( (WIDTH/5)+(rand()%(3*WIDTH/5)), (int(ymin)/4)+(rand()%(int(ymin)/2)) );
	sp = new Sprite(*wizardAS);
	sp->PlayAnimation("idle");
	sp->SetNextAnimation("idle");
	wizard1->addSprite(0, sp);

	wizard2 = new Wizard( (WIDTH/5)+(rand()%(3*WIDTH/5)), (int(ymax)+((HEIGHT-int(ymax))/5))+(rand()%(3*(HEIGHT-int(ymax))/5)) );
	sp = new Sprite(*wizardAS);
	sp->PlayAnimation("idle");
	sp->SetNextAnimation("idle");
	wizard2->addSprite(0, sp);

	curWizard   = wizard1;
	otherWizard = wizard2;


	state = Gesture;
}

static GLenum getGlFormat(int nComponents)
{
	switch (nComponents)
	{
	case 1: return GL_LUMINANCE;
	case 3: return GL_RGB;
	case 4: return GL_RGBA;
	default:
		fprintf(stderr, "Invalid number of color components\n");
		exit(-1);
	}
}


void Update(double timeDif)
{
	

	static float y = HEIGHT;
	y += -200.0f*(float)timeDif;

	// Move shot
	if (shot)
	{
		if (state == Shot)
		{
			
			int mx, my;
			Uint8 button = SDL_GetMouseState(&mx, &my);
			if (button & SDL_BUTTON_LEFT)
			{
				if (curWizard->hasMana())
				{
					curWizard->decMana(timeDif*COST_MANIP);
					if (curWizard->hasMana())
						shot->Manipulate(mx, my, timeDif);
					else
						PlaySoundEffect("needmana");
				}
			}
		}

		float shotW2 = 32/2;
		float shotH2 = 32/2;
		shot->Update(timeDif);


		// Check shot over spell source
		for (SpellSourceIt it = spellSources.begin(); it != spellSources.end(); ++it)
		{
			// Check if already used source
			if (spellSourcesUsed.find((*it).second) != spellSourcesUsed.end())
				continue;

			BBox& bbox = ((*it).second)->getBBox();
			if (bbox.pointInBox(shot->pos[0], shot->pos[1]))
			{
				// TODO: Add effect here
				eSpellReaction react = ((*it).second)->affectShot(*shot);
				SpellPoints& pts = curWizard->getSpellPts();
				pts += ((*it).second)->getWizardSP();
				spellSourcesUsed.insert((*it).second);

				if (react == eShotFizzle)
				{
					PlaySoundEffect("fizzle");
					delete shot;
					shot = 0;
					NextPlayer();
					break;
				}
			}
		}

		if (shot)
		{
			// Check if hit other player
			float wx, wy;
			BBox& bbox = otherWizard->getBBox();
			if (bbox.pointInBox(shot->pos[0],shot->pos[1]))
			{
				int damage = shot->calcDamage();
				otherWizard->decHps(damage);

				bool generic = shot->type == eGeneric;

				delete shot;
				shot = 0;

				if (otherWizard->isDead())
				{			
					state = Death;
					Sprite* sp = otherWizard->getSprite(0);
					sp->PlayAnimation("die");
					sp->ClearNextAnimation();

					if (rand() % 2)
						PlaySoundEffect("w_die1");
					else
						PlaySoundEffect("w_die2");

					// sp->SetNextAnimation("dead");
				} else {

					if (generic)
						PlaySoundEffect("fizzle");
					else
					{
						int hit = rand() % 4;
						switch (hit)
						{
						case 0: PlaySoundEffect("w_hit1"); break;
						case 1: PlaySoundEffect("w_hit2"); break;
						case 2: PlaySoundEffect("w_hit3"); break;
						case 3: PlaySoundEffect("w_hit4"); break;
						}

						if (rand() % 2)
							PlaySoundEffect("hit1");
						else
							PlaySoundEffect("hit2");

						Sprite* sp = otherWizard->getSprite(0);
						sp->PlayAnimation("hit");
						sp->SetNextAnimation("idle");
					}

					NextPlayer();
				}

			} else if (shot->pos[0] < -shotW2*10 || shot->pos[0] >  WIDTH+shotW2*10 ||
				shot->pos[1] < -shotH2*10 || shot->pos[1] > HEIGHT+shotH2*10 || shot->life < 0)
			{
			// Check if shot offscreen
				PlaySoundEffect("w_miss");
				delete shot;
				shot = 0;
				NextPlayer();
			}

		}
	}

	// Move particles
	/* emitter->x = (float)WIDTH/2.0;
	emitter->y = y;
	emitter->z = 0.1f;*/


	// Move all particles
	emitterSet.Update(timeDif);

	clouds->Update(timeDif);


	// update spell sources
	//
	SpellSourceIt ssIter;
	for( ssIter = spellSources.begin(); ssIter != spellSources.end(); ssIter++ )
	{
		((*ssIter).second)->update(timeDif);
	}


	// update wand emitter
	if (state == Create && wandEmitter)
	{
		wandEmitter->x = mousePos[0];
		wandEmitter->y = mousePos[1];
	}

	// update wizards
	//
	wizard1->update(timeDif);
	wizard2->update(timeDif);

	if( state == Walk && !curWizard->isWalking() )
	{
		state = Gesture;

		curWizard->setHasWalked();

		Sprite* sp = curWizard->getSprite(0);
		sp->PlayAnimation("idle");
		sp->SetNextAnimation("idle");
	}

	// Update animations
	if (state == Death)
	{
		Sprite* sp = otherWizard->getSprite(0);
		if (!sp->IsPlaying())
		{
			Sprite* csp = curWizard->getSprite(0);
			csp->PlayAnimation("celebrate");
			csp->ClearNextAnimation();
			if (rand() % 2)
				PlaySoundEffect("victory1");
			else
				PlaySoundEffect("victory2");
			state = Victory;
		}
	} else if (state == Victory) {
		Sprite* sp = curWizard->getSprite(0);
		if (!sp->IsPlaying())
		{
			exit(0);
		}
	}

/*	Sprite* sprite;
	sprite = wizard1->getSprite(0);
	if( sprite )
	{
		static float a  = 1.0f;
		static float da = -0.1f;

		if( !sprite->IsPlaying() )
		{
			sprite->PlayAnimation("redAnim");
		}

		a += (da * timeDif);
		if( a < 0.1 ) da =  0.1f;
		if( a > 0.9 ) da = -0.1f;
		sprite->SetAlpha(a);
	}
	wizard1->update(timeDif);

	sprite = wizard2->getSprite(0);
	if( sprite && !sprite->IsPlaying() ) sprite->PlayAnimation("whiteAnim");
	wizard2->update(timeDif); */
}


void DrawBar(const Frame* frame, float val, int x, int y)
{
	frame->Draw(x, y, 0, 0.25, 0.25, 0.3, 1.0, 1.0, 1.0, false, false);
	frame->DrawPercent(x, y, 0, 0.25, 0.25, 1.0, 1.0, 1.0, 1.0, val);

}

void DrawInterfaceWizard(float tl[2], float spacing, Wizard* w)
{
	SpellPoints& pts = w->getSpellPts();
	DrawBar(healthHudFrame, (float)w->getHps() / MAX_WIZ_HPS, tl[0], tl[1]);
	DrawBar(manaHudFrame, (float)w->getMana() / MAX_WIZ_MANA, tl[0], tl[1] + spacing);

	DrawBar(fireHudFrame, (float)pts.getFirePts() / MAX_SP_F, tl[0], tl[1] + spacing*3);
	DrawBar(waterHudFrame, (float)pts.getWaterPts() / MAX_SP_W, tl[0], tl[1] + spacing*4);
	DrawBar(earthHudFrame, (float)pts.getEarthPts() / MAX_SP_E, tl[0], tl[1] + spacing*5);
	DrawBar(elecHudFrame, (float)pts.getElecPts() / MAX_SP_L, tl[0], tl[1] + spacing*6);

	// Draw text
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
	glEnable(GL_TEXTURE_2D);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	

	char *str;

	int yText;

	if (curWizard == wizard1)
	{
		glColor4f(1.0f, 0.0f, 0.5f,1);
		str = "Player 1's turn";
		yText = 16;
	}
	else
	{
		glColor4f(0.0f, 1.0f, 0.5f,1);
		str = "Player 2's turn";
		yText = HEIGHT-16-32;
	}

	font.Begin();
	font.TextOut(str, WIDTH/2 - 100, yText, 0);
	glPopMatrix();
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
}

void DrawInterface()
{
	float spacing = 40;
	float tl1[2] = {32, 32};
	float tl2[2] = {WIDTH-32, HEIGHT-spacing*7};
	DrawInterfaceWizard(tl1, spacing, wizard1);
	DrawInterfaceWizard(tl2, spacing, wizard2);

	Uint8 *keys = SDL_GetKeyState(NULL);

	if(keys['h'] || keys[SDLK_F1])
	{
		// Draw help
		helpFrame->Draw(WIDTH/2, HEIGHT/2, 0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, false, false);
	}

}

void Draw()
{
	SDL_ShowCursor(0);
	// Draw scene
	glClear(GL_COLOR_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	
	// Draw background	
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_DECAL);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texBack);
	glColor4f(1.0, 1.0, 1.0, 1.0);
	glBegin(GL_QUADS);
		glTexCoord2f(0, 1);
		glVertex3f(0, (float)HEIGHT, 0);
		glTexCoord2f(1, 1);
		glVertex3f( (float)WIDTH, (float)HEIGHT, 0.0);
		glTexCoord2f(1, 0);
		glVertex3f( (float)WIDTH,  0, 0.0);
		glTexCoord2f(0, 0);
		glVertex3f(0,  0, 0.0);
	glEnd();
	glDisable(GL_TEXTURE_2D);


	// draw spell sources and wizards
	//
	std::pair<float, Entity*> entity;
	std::multimap<float, Entity*> entities;

	SpellSourceIt ssIter;
	for( ssIter = spellSources.begin(); ssIter != spellSources.end(); ssIter++ )
	{
		entity.first  = (*ssIter).first;
		entity.second = (*ssIter).second;

		entities.insert(entity);
	}

	Frame * arm = NULL;
	wizardAS->GetFrame("arm",arm);
	float x, y;
	wizard1->getPosition(x, y);
	entity.first  = y;
	entity.second = wizard1;
	entities.insert(entity);
	float angle;

	bool drawarm = false;
	Sprite *wizSprite = wizard1->getSprite(0);
	int armx = 0;
	int army = 0;
	if (wizSprite && (curWizard == wizard1) && (wizSprite->GetAnimName() == "idle"))
	{
		drawarm = true;
		angle = atan2f(x-mousePos[0],mousePos[1]-y)*(180.0/3.14159);
		armx = x;
		army = y;
	}


	wizard2->getPosition(x, y);
	entity.first  = y;
	entity.second = wizard2;
	entities.insert(entity);

	wizSprite = wizard2->getSprite(0);

	if (wizSprite && (curWizard == wizard2) && (wizSprite->GetAnimName() == "idle"))
	{
		drawarm = true;
		angle = atan2f(x-mousePos[0],mousePos[1]-y)*(180.0/3.14159);
		armx = x;
		army = y;
	}


	std::multimap<float, Entity*>::iterator eIter;
	bool armdrawn = false;
	for( eIter = entities.begin(); eIter != entities.end(); eIter++ )
	{
		if (!armdrawn && (eIter->first >= army) && arm && drawarm)
		{
			arm->Draw(armx,army-64,angle,1,1,1,1,1,1,false,false);
			armdrawn = true;
		}
		((*eIter).second)->draw();

	}


	// Draw shot
	if (shot)
		shot->Draw();

	// Draw particles
	emitterSet.Draw();


	// Draw clouds
	clouds->Draw();


	// Draw interface
	DrawInterface();

	if (state == Victory) {
		victoryFrame->Draw(WIDTH/2, HEIGHT/2, 0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, false, false);
	}

	//Draw the curser
	const Frame * cframe = hudfs.GetFrame("wand");
	float wandrot = 0;
	
	if (cframe)
		cframe->Draw(mousePos[0],mousePos[1],mouseVel[0]-45,0.4,0.4,1,1,1,1,false,false);
		
	SDL_GL_SwapBuffers();

	timeLast = timeCur;
}
