#ifndef particle_h
#define particle_h

#include <list>

struct Particle
{
	float pos[3];
	float col[3];
	float vel[3];
	float life;
	float fade;
};

class ParticleEmitter;

class ParticleEmitterSet
{
public:
	void Draw();
	void Update(double timeDif);
	void Add(ParticleEmitter* e);

private:
	typedef std::list<ParticleEmitter*> EmitterCont;
	typedef EmitterCont::iterator EmitterIt;
	EmitterCont emitters;
};

extern ParticleEmitterSet emitterSet;


class ParticleEmitter
{
public:
	ParticleEmitter(unsigned int texId, double w, double h, double tpp, double ttl, float col1[3], float col2[3]);

	bool Update(double time);
	virtual bool UpdateParticle(Particle& p, double timeElapsed);

	void Draw();

	virtual Particle CreateParticle(float x, float y, float z);

	void Kill();

	float x, y, z;

	virtual ParticleEmitter* Clone();

protected:
	float lastPos[3];
	double curTime;

private:
	bool   killed;
	double timePerParticle;
	double timeToLive;

	double timeSinceLast;

	unsigned int tex;
	double width, height;
	float color1[3], color2[3];

	typedef std::list<Particle> ParticleCont;
	typedef ParticleCont::iterator ParticleIt;
	ParticleCont particles;
};


class SpinEmitter : public ParticleEmitter
{
public:
	SpinEmitter(unsigned int texId, double w, double h, double tpp, double ttl, float col1[3], float col2[3], float r, float freq);
	virtual Particle CreateParticle(float x, float y, float z);
	virtual ParticleEmitter* Clone();

private:
	float radius;
	float frequency;
};



#endif