#include <math.h>

#include "wizard.h"


Wizard::Wizard(float x, float y) :
	Entity(x, y, DEF_ENTITY_W, 1.25f*DEF_ENTITY_H),
	hps(MAX_WIZ_HPS),
	mana(MAX_WIZ_MANA),
	speed(WIZ_WALK_SPEED),
	hasMoved(false),
	spellPts(1,1,1,1)
{
	moveTo[0] = pos[0];
	moveTo[1] = pos[1];

	dir[0] = dir[1] = 0.0f;
}

Wizard::~Wizard()
{
}

float
Wizard::getHps() const
{
	return hps;
}

void
Wizard::setHps(float h)
{
	hps = h;

	if( hps < MIN_WIZ_HPS ) hps = MIN_WIZ_HPS;
	if( hps > MAX_WIZ_HPS ) hps = MAX_WIZ_HPS;
}

float
Wizard::incHps(float dhps)
{
	hps += dhps;

	if( hps < MIN_WIZ_HPS ) hps = MIN_WIZ_HPS;
	if( hps > MAX_WIZ_HPS ) hps = MAX_WIZ_HPS;

	return hps;
}

float
Wizard::decHps(float dhps)
{
	hps -= dhps;

	if( hps < MIN_WIZ_HPS ) hps = MIN_WIZ_HPS;
	if( hps > MAX_WIZ_HPS ) hps = MAX_WIZ_HPS;

	return hps;
}

bool
Wizard::isDead() const
{
	return (!(hps > MIN_WIZ_HPS));
}


float
Wizard::getMana() const
{
	return mana;
}

void
Wizard::setMana(float m)
{
	mana = m;

	if( mana < MIN_WIZ_MANA ) mana = MIN_WIZ_MANA;
	if( mana > MAX_WIZ_MANA ) mana = MAX_WIZ_MANA;
}

float
Wizard::incMana(float dmana)
{
	mana += dmana;

	if( mana < MIN_WIZ_MANA ) mana = MIN_WIZ_MANA;
	if( mana > MAX_WIZ_MANA ) mana = MAX_WIZ_MANA;

	return mana;
}

float
Wizard::decMana(float dmana)
{
	mana -= dmana;

	if( mana < MIN_WIZ_MANA ) mana = MIN_WIZ_MANA;
	if( mana > MAX_WIZ_MANA ) mana = MAX_WIZ_MANA;

	return mana;
}

bool
Wizard::hasMana() const
{
	return (mana > MIN_WIZ_MANA);
}

void
Wizard::getMoveTo(float& x, float& y) const
{
	x = moveTo[0];
	y = moveTo[1];
}

void
Wizard::setMoveTo(float  x, float  y)
{
	moveTo[0] = x;
	moveTo[1] = y;

	dir[0] = moveTo[0] - pos[0];
	dir[1] = moveTo[1] - pos[1];

	float d = sqrt(dir[0]*dir[0]+dir[1]*dir[1]);

	dir[0] = (dir[0] / d) * speed;
	dir[1] = (dir[1] / d) * speed;
}

void
Wizard::getDir(float& x, float& y) const
{
	x = dir[0];
	y = dir[1];
}

bool
Wizard::isWalking() const
{
	return (!((fabs(moveTo[0] - pos[0]) < CONST_FLOAT_ZERO) && (fabs(moveTo[1] - pos[1]) < CONST_FLOAT_ZERO)));
}

bool
Wizard::hasWalked() const
{
	return hasMoved;
}

void
Wizard::setHasWalked()
{
	hasMoved = true;
}

void
Wizard::resetHasWalked()
{
	hasMoved = false;
}

void
Wizard::update(double dt)
{
	Entity::update(dt);

	float x, y;
	x = pos[0] + (dt*dir[0]);
	y = pos[1] + (dt*dir[1]);

	setPosition(x, y);

	if( (fabs(x - moveTo[0]) < CONST_PIXEL_SIZE) &&
	    (fabs(y - moveTo[1]) < CONST_PIXEL_SIZE) )
	{
		moveTo[0] = pos[0];
		moveTo[1] = pos[1];

		dir[0] = dir[1] = 0.0f;
	}
}
